<?php
/**
* File class
* 
* File identifica un preciso file nel filesystem e permette
* di eseguire le pi comuni operazioni su di esso, oltre a un
* particolare sistema di locking.
* 
* @package Filesystem
* @author  Alex Dupre <sysadmin@alexdupre.com>
* @version $Id: File.php,v 1.6 2002/08/23 13:57:58 alexdupre Exp $
*/
class File
{
    /**
    * File path
    * 
    * @access private
    * @var string $path
    * @see File()
    */
    var $path;
    
    /**
    * File handle
    * 
    * @access private
    * @var int $fp
    */
    var $fp;
    
    /**
    * Class constructor
    * 
    * Identifica il file su cui operare.
    * 
    * @param string $path path del file
    */
    function File($path)
    {
        $this->path = $path;
    }
    
    /**
    * Chiude il file
    * 
    * @access public
    * @return boolean TRUE se la chiusura ha avuto successo, FALSE altrimenti
    */
    function close()
    {
        if (!$this->isOpen()) return TRUE;
        return @fclose($this->fp);
    }
    
    /**
    * Cancella il file
    * 
    * @access public
    * @return boolean TRUE se la cancellazione ha avuto successo, FALSE altrimenti
    */
    function delete()
    {
        if (!$this->exist()) return TRUE;
        return @unlink($this->path);
    }
    
    /**
    * Controlla se il file esiste
    * 
    * @access public
    * @return boolean TRUE se il file esiste, FALSE altrimenti
    */
    function exist()
    {
        return @file_exists($this->path);
    }
    
    /**
    * Legge il file e lo restituisce come array
    * 
    * Questa funzione ha senso solo con files di testo,
    * dove ogni riga ocuuper una posizione differente nell'array.
    * 
    * @access public
    * @return array  array contenente le righe del file
    */
    function getArray()
    {
        return @file($this->path);
    }
    
    /**
    * Legge il file e lo restituisce come stringa
    * 
    * @access public
    * @return string contenuto del file
    */
    function getString()
    {
        if ($this->isOpen()) $this->goToStart();
        elseif (!$this->open()) return FALSE;
        return @fread($this->fp, $this->size());
    }
    
    /**
    * Sposta il puntatore alla fine del file
    * 
    * @access public
    * @return boolean TRUE se lo spostamento ha avuto successo, FALSE altrimenti
    */
    function goToEnd()
    {
        if (!$this->isOpen()) return FALSE;
        return (@fseek($this->fp, 0, SEEK_END) == 0) ? TRUE : FALSE;
    }
    
    /**
    * Sposta il puntatore all'inizio del file
    * 
    * @access public
    * @return boolean TRUE se lo spostamento ha avuto successo, FALSE altrimenti
    */
    function goToStart()
    {
        if (!$this->isOpen()) return FALSE;
        return @rewind($this->fp);
    }
    
    /**
    * Controlla se il file  aperto
    * 
    * @access public
    * @return boolean TRUE se il file  aperto, FALSE altrimenti
    */
    function isOpen()
    {
        return ($this->fp) ? TRUE : FALSE;
    }
    
    /**
    * Esegue il lock del file
    * 
    * Il file dev'essere gi stato aperto.
    * Il lock funziona solo se tutti i processi che accedono al file stesso
    * utilizzano queste funzioni di lock.
    * Il lock non ha effetto se il file si trova su uno share NFS.
    * 
    * @access public
    * @param  boolean $write TRUE per un lock in scrittura (exclusive), FALSE in lettura (shared)
    * @return boolean TRUE se il lock ha avuto successo, FALSE altrimenti
    */
    function lock($write = FALSE)
    {
        if (!$this->isOpen()) return FALSE;
        return @flock($this->fp, ($write) ? LOCK_EX : LOCK_SH);
    }
    
    /**
    * Controlla la data dell'ultima modifica
    * 
    * @access public
    * @return integer timestamp con l'ultima modifica al file
    */
    function mtime()
    {
        return @filemtime($this->path);
    }
    
    /**
    * Apre il file
    * 
    * Il file pu essere aperto in diverse modalit:
    * - r  : lettura
    * - r+ : lettura e scrittura
    * - w  : scrittura (il file verr troncato)
    * - w+ : scrittura e lettura (il file verr troncato)
    * - a  : scrittura alla fine del file
    * - a+ : scrittura e lettura alla fine del file
    * 
    * @access public
    * @param  string  $mode modalit di apertura del file ('r', 'r+', 'w', 'w+', 'a', 'a+')
    * @return boolean TRUE se l'apertura ha avuto successo, FALSE altrimenti
    */
    function open($mode = 'r')
    {
        $this->fp = @fopen($this->path, $mode . 'b');
        return $this->isOpen();
    }
    
    /**
    * Aggiunge una stringa al file in modo sicuro (utilizzando il meccanismo di lock)
    * 
    * @access public
    * @param  string  $str stringa da aggiungere
    * @return boolean TRUE se l'operazione ha avuto successo, FALSE altrimenti
    */
    function safeAppend($str)
    {
        return ($this->open('a') && $this->lock() &&
                $this->setString($str) &&
                $this->unlock() && $this->close());
    }
    
    /**
    * Legge il file in modo sicuro (utilizzando il meccanismo di lock)
    * 
    * @access public
    * @return string il contenuto del file
    */
    function safeRead()
    {
        if (!($this->open() && $this->lock())) return FALSE;
        $data = $this->getString();
        if (!($this->unlock() && $this->close())) return FALSE;
        return $data;
    }
    
    /**
    * Aggiorna/crea il file in modo sicuro (utilizzando il meccanismo di lock)
    * 
    * @access public
    * @param  string  $str (nuovo) contenuto del file
    * @return boolean TRUE se l'operazione ha avuto successo, FALSE altrimenti
    */
    function safeWrite($str)
    {
        return (($this->open('r+') || $this->open('w')) && $this->lock() &&
                $this->setString($str) && $this->truncate() &&
                $this->unlock() && $this->close());
    }

    /**
    * Salva il contenuto dell'array nel file
    * 
    * @access public
    * @param  array   $rows   array con il contenuto da salvare
    * @param  string  $union  caratteri da inserire nell'unione delle righe
    * @param  boolean $append TRUE per sovrascrivere il file, FALSE per aggiungere le righe alla fine
    * @return boolean TRUE se l'operazione ha avuto successo, FALSE altrimenti
    */
    function setArray($rows, $union = '', $append = FALSE)
    {
        return $this->setString(join($union, $rows), $append);
    }
    
    /**
    * Salva il contenuto della stringa nel file
    * 
    * @access public
    * @param  string  $str    stringa con il contenuto da salvare
    * @param  boolean $append TRUE per sovrascrivere il file, FALSE per aggiungere la stringa alla fine
    * @return boolean TRUE se l'operazione ha avuto successo, FALSE altrimenti
    */
    function setString($str, $append = FALSE)
    {
        if ($this->isOpen()) ($append) ? $this->goToEnd() : $this->goToStart();
        else $this->open(($append) ? 'a' : 'w');
        return @fwrite($this->fp, $str);
    }
    
    /**
    * Trova la dimensione del file
    * 
    * @access public
    * @return integer dimensione del file (in bytes)
    */
    function size()
    {
        return @filesize($this->path);
    }
    
    /**
    * Tronca il file alla posizione corrente
    * 
    * @access public
    * @return boolean TRUE se il troncamento ha avuto successo, FALSE altrimenti
    */
    function truncate()
    {
        if (!$this->isOpen() && !$this->open('r+')) return FALSE;
        return @ftruncate($this->fp, @ftell($this->fp));
    }
    
    /**
    * Esegue l'unlock del file
    * 
    * @access public
    * @return boolean TRUE se l'unlock ha avuto successo, FALSE altrimenti
    */
    function unlock()
    {
        if (!$this->isOpen()) return FALSE;
        return @flock($this->fp, LOCK_UN);
    }
}
?>
